@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 # Copyright (c) 2018 Stephen Wasilewski
 # =======================================================================
 # This Source Code Form is subject to the terms of the Mozilla Public
@@ -8,7 +10,9 @@
 """console script for graphing data."""
 
 from __future__ import print_function
+from __future__ import division
 
+from past.utils import old_div
 from clasp import click
 import matplotlib.colors as mplc
 
@@ -21,11 +25,12 @@ import clipt.plot as ruplot
 
 @click.group()
 @clk.shared_decs(clk.main_decs(__version__))
-def main(ctx, config, outconfig, configalias, inputalias):
-    """help configuration and process management commands."""
+def plot(ctx, config, outconfig, configalias, inputalias):
+    """scripts for plotting data from the command line."""
     template = get_root() + "/templates/master.cfg"
     clk.get_config(ctx, config, outconfig, configalias, inputalias, template)
 
+main = plot # backwards compatibility
 
 # used by all
 shared = [
@@ -131,7 +136,7 @@ sharedB = [
 ]
 
 
-@main.command('bar')
+@plot.command('bar')
 @click.argument('dataf', callback=clk.are_files)
 @click.option('-rwidth', default=0.95, type=float,
               help="relative width of bar clusters")
@@ -223,7 +228,7 @@ def bar(ctx, dataf, **kwargs):
     return 'bar', kwargs, ctx
 
 
-@main.command('heatmap')
+@plot.command('heatmap')
 @click.argument('dataf', callback=clk.is_file)
 @click.option('-pph', default=1, type=int,
               help="data points per hour in annual data")
@@ -293,7 +298,7 @@ def heatmap(ctx, dataf, **kwargs):
     return 'heatmap', kwargs, ctx
 
 
-@main.command('histo')
+@plot.command('histo')
 @click.argument('dataf', callback=clk.are_files)
 @click.option('-y_vals', default="-1", callback=clk.tup_int,
               help="index for yvals")
@@ -385,7 +390,7 @@ def histo(ctx, dataf, **kwargs):
     return 'histo', kwargs, ctx
 
 
-@main.command('scatter')
+@plot.command('scatter')
 @click.argument('dataf', callback=clk.are_files)
 @click.option('-autox', default=None,
               callback=clk.split_float,
@@ -420,6 +425,10 @@ def histo(ctx, dataf, **kwargs):
               "min and max of data enter xmin etc to maintain autoscale")
 @click.option('--xlog/--no-xlog', default=False,
               help="plot x on log scale")
+@click.option('--diagonal/--no-diagonal', default=False,
+              help="add diagonal gridline")
+@click.option('--matchxy/--no-matchxy', default=False,
+              help="use same range for x and y")
 @click.option('-pery', default=None, type=int,
               help="label y-axis by percentile with N bins")
 @click.option('--reverse/--no-reverse', default=False,
@@ -476,13 +485,18 @@ def scatter(ctx, dataf, **kwargs):
             emap = ruplot.get_colors(kwargs['ecolors'], **a5)
             a6 = mgr.kwarg_match(ruplot.plot_scatter, kwargs)
             a6.pop('labels', None)
-            ax, handles = ruplot.plot_scatter(ax, xs, ys, labels, cmap,
+            a6['legend'] = kwargs['legend'] and not kwargs['areaonly']
+            ax, handles = ruplot.plot_scatter(fig, ax, xs, ys, labels, cmap,
                                               emap=emap, cs=cs, msd=msd, **a6)
+            if kwargs['diagonal']:
+                ax.plot(ax.get_xlim(), ax.get_ylim(), lw=0.6, ls='--', color='black', ms=0)
             if kwargs['outf']:
                 outf = kwargs['outf']
             else:
                 outf = dataf[0].rsplit(".", 1)[0] + ".png"
             a7 = mgr.kwarg_match(ruplot.plot_graph, kwargs)
+            if cs is not None:
+                a7['legend'] = False
             ruplot.plot_graph(fig, outf, handles=handles, **a7)
         except click.Abort:
             raise
@@ -491,7 +505,7 @@ def scatter(ctx, dataf, **kwargs):
     return 'scatter', kwargs, ctx
 
 
-@main.command('box')
+@plot.command('box')
 @click.argument('dataf', callback=clk.are_files)
 @click.option('-y_vals', default="-1", callback=clk.tup_int,
               help="index for yvals")
@@ -499,26 +513,66 @@ def scatter(ctx, dataf, **kwargs):
               help="enter as xname,xmin,xmax,yname,ymin,ymax - default uses"
               "min and max of data enter xmin etc to maintain autoscale")
 @click.option('-rwidth', default=0.95, type=float,
-              help="relative width of bars")
+              help="relative width of boxs")
 @click.option('-xlabels', callback=clk.split_str,
               help="input custom xaxis labels, by default uses file name and"
               "row number or --xheader option")
+@click.option('-mark', default='x',
+              help="mark for flyers")
+@click.option('-ms', default=3.0,
+              help="size for flyers")
+@click.option('-mew', default=.5,
+              help="edge weight for flyers")
+@click.option('-lw', default=1.,
+              help="linewidth for box and whiskers")
+@click.option('-clw', default=1.,
+              help="linewidth for median line")
+@click.option('-series', default=1,
+              help="number of series to plot data in")
+@click.option('--clbg/--no-clbg', default=True,
+              help="median line uses -bg")
+@click.option('-fillalpha', default=1.0,
+              help="alpha for fill color (matches line color)")
 @click.option('--xheader/--no-xheader', default=False,
               help="indicates that data has a header column to get x-axis "
               "labels (overridden by xlabels)")
-@clk.shared_decs(shared + sharedA)
+@click.option('-fcol', default=0.0, type=float,
+              help="colormap position for first color")
+@click.option('--xgrid/--no-xgrid', default=False,
+              help="plot x grid lines")
+@click.option('--ygrid/--no-ygrid', default=False,
+              help="plot y grid lines")
+@click.option('-yticks', type=int,
+              help="number of y-ticks/gridlines")
+@click.option('-labels', callback=clk.split_str,
+              help="input custom x-axis labels, by default uses "
+              "file name and index or --header option")
+@click.option('--rows/--no-rows', default=False,
+              help="get data rows instead of columns")
+@click.option('--ylog/--no-ylog', default=False,
+              help="plot y on log scale")
+@click.option('--mean/--median', default=False,
+              help="plot mean line or median line")
+@click.option('--notch/--no-notch', default=False,
+              help="plot notch for 95\% confidence interval")
+@click.option('--inline/--no-inline', default=False,
+              help="keep boxes inline")
+@click.option('--header/--no-header', default=False,
+              help="indicates that data has a header row to get "
+              "series labels (overridden by labels)")
+@clk.shared_decs(shared)
 def box(dataf, **kwargs):
     """
     create boxplot from data files.
     """
     if kwargs['opts']:
         kwargs['opts'] = False
-        clk.echo_args(arg1, **kwargs)
+        clk.echo_args(dataf, **kwargs)
     else:
         try:
             axext = ruplot.get_axes(kwargs['axes'], [], [])
             a1 = mgr.kwarg_match(mgr.read_data, kwargs)
-            a1['autox'] = axext['xdata']
+            a1['autox'] = [0, 1]
             xs, ys, labels = mgr.read_all_data(dataf, **a1)
             if kwargs['xheader']:
                 a1['y_vals'] = [0]
@@ -541,7 +595,7 @@ def box(dataf, **kwargs):
             a4 = mgr.kwarg_match(ruplot.ticks, kwargs)
             a4.pop('labels', None)
             a4['xlabels'] = xlabels
-            ax = ruplot.tick_from_arg(ax, xs, ys, a4, kwargs)
+            ax = ruplot.tick_from_arg(ax, [0, len(ys)], ys, a4, kwargs)
             a5 = mgr.kwarg_match(ruplot.get_colors, kwargs)
             cmap = ruplot.get_colors(kwargs['colors'], **a5)
             a6 = mgr.kwarg_match(ruplot.plot_box, kwargs)
@@ -561,7 +615,113 @@ def box(dataf, **kwargs):
     return 'box', kwargs
 
 
-@main.command('colors')
+@plot.command('violin')
+@click.argument('dataf', callback=clk.are_files)
+@click.option('-y_vals', default="-1", callback=clk.tup_int,
+              help="index for yvals")
+@click.option('-axes', default="X,0,1,Y,0,ymax",
+              help="enter as xname,xmin,xmax,yname,ymin,ymax - default uses"
+              "min and max of data enter xmin etc to maintain autoscale")
+@click.option('-rwidth', default=0.95, type=float,
+              help="relative width of boxs")
+@click.option('-xlabels', callback=clk.split_str,
+              help="input custom xaxis labels, by default uses file name and"
+              "row number or --xheader option")
+@click.option('-lw', default=1.,
+              help="linewidth for extrema and vertical line")
+@click.option('-clw', default=1.,
+              help="linewidth for median line")
+@click.option('-series', default=1,
+              help="number of series to plot data in")
+@click.option('--clbg/--no-clbg', default=True,
+              help="median line uses -bg")
+@click.option('-fillalpha', default=1.0,
+              help="alpha for fill color (matches line color)")
+@click.option('--xheader/--no-xheader', default=False,
+              help="indicates that data has a header column to get x-axis "
+              "labels (overridden by xlabels)")
+@click.option('-fcol', default=0.0, type=float,
+              help="colormap position for first color")
+@click.option('--xgrid/--no-xgrid', default=False,
+              help="plot x grid lines")
+@click.option('--ygrid/--no-ygrid', default=False,
+              help="plot y grid lines")
+@click.option('-yticks', type=int,
+              help="number of y-ticks/gridlines")
+@click.option('-labels', callback=clk.split_str,
+              help="input custom x-axis labels, by default uses "
+              "file name and index or --header option")
+@click.option('--rows/--no-rows', default=False,
+              help="get data rows instead of columns")
+@click.option('--ylog/--no-ylog', default=False,
+              help="plot y on log scale")
+@click.option('--mean/--no-mean', default=False,
+              help="plot mean dot")
+@click.option('--median/--no-median', default=True,
+              help="plot median line")
+@click.option('--inline/--no-inline', default=False,
+              help="keep violins inline")
+@click.option('--header/--no-header', default=False,
+              help="indicates that data has a header row to get "
+              "series labels (overridden by labels)")
+@clk.shared_decs(shared)
+def violin(dataf, **kwargs):
+    """
+    create violinplot from data files.
+    """
+    if kwargs['opts']:
+        kwargs['opts'] = False
+        clk.echo_args(dataf, **kwargs)
+    else:
+        try:
+            axext = ruplot.get_axes(kwargs['axes'], [], [])
+            a1 = mgr.kwarg_match(mgr.read_data, kwargs)
+            a1['autox'] = [0, 1]
+            xs, ys, labels = mgr.read_all_data(dataf, **a1)
+            if kwargs['xheader']:
+                a1['y_vals'] = [0]
+                a1['x_vals'] = []
+                a1['coerce'] = False
+                a1['xheader'] = False
+                a1['rows'] = False
+                _, xlabels, _ = mgr.read_data(dataf[0], **a1)
+                xlabels = xlabels[0]
+            else:
+                xlabels = []
+            xlabels = ruplot.get_labels(xlabels, kwargs['xlabels'])
+            labels = ruplot.get_labels(labels, kwargs['labels'])
+            if kwargs['rows']:
+                la = labels
+                labels = xlabels
+                xlabels = la
+            a3 = mgr.kwarg_match(ruplot.plot_setup, kwargs)
+            ax, fig = ruplot.plot_setup(**a3)
+            a4 = mgr.kwarg_match(ruplot.ticks, kwargs)
+            a4.pop('labels', None)
+            a4['xlabels'] = xlabels
+            ax = ruplot.tick_from_arg(ax, [0, len(ys)], ys, a4, kwargs)
+            a5 = mgr.kwarg_match(ruplot.get_colors, kwargs)
+            cmap = ruplot.get_colors(kwargs['colors'], **a5)
+            a6 = mgr.kwarg_match(ruplot.plot_box, kwargs)
+            a6.pop('labels', None)
+            ax, handles = ruplot.plot_violin(ax, ys, labels, cmap,
+                                             axext['ydata'], **a6)
+            if kwargs['outf']:
+                outf = kwargs['outf']
+            else:
+                outf = dataf[0].rsplit(".", 1)[0] + ".png"
+            a7 = mgr.kwarg_match(ruplot.plot_graph, kwargs)
+            ruplot.plot_graph(fig, outf, handles=handles, **a7)
+        except click.Abort:
+            raise
+        except Exception as ex:
+            clk.print_except(ex, kwargs['debug'])
+    return 'violin', kwargs
+
+
+
+
+@plot.command('colors')
 @click.option('-outf',
               help="output file")
 @click.option('--ru/--mat', default=True,
@@ -570,6 +730,10 @@ def box(dataf, **kwargs):
               help="print rgb values for radutil")
 @click.option('--radf/--no-radf', default=False,
               help="print rgb values on 0-1 scale")
+@click.option('-mpal',
+              help="matplotlib palette to print r,g,b vals")
+@click.option('-mpaldiv', default=10,
+              help="number of segments")
 @click.option('--opts', '-opts', is_flag=True,
               help="check parsed options")
 @click.option('--debug', is_flag=True,
@@ -609,9 +773,21 @@ def colors(**kwargs):
                                                    for i in rucol.shades]))
                     for k in rucol.colors:
                         v = rucol.cdict255[k]
-                        fmt = [" ".join(["{:06f}".format(j/255.**2.2)
+                        fmt = [" ".join(["{:06f}".format(old_div(j,255.)**2.2)
                                          for j in i]) for i in v]
                         click.echo("{}:  {}".format(k, "  ".join(fmt)))
+            if kwargs['mpal'] is not None:
+                cmap = ruplot.get_colors(kwargs['mpal'])
+                click.echo("\ncolor palette: {} in {} steps:".format(kwargs['mpal'], kwargs['mpaldiv']))
+                for i in range(kwargs['mpaldiv']):
+                    c = cmap.to_rgba(i/(kwargs['mpaldiv'] - 1.0))
+                    click.echo("{:03d},{:03d},{:03d}".format(*[int(j*255) for j in c[0:3]]))
+                if kwargs['radf']:
+                    click.echo("\n ungammad 0-1:")
+                    for i in range(kwargs['mpaldiv']):
+                        c = cmap.to_rgba(i/(kwargs['mpaldiv'] - 1.0))
+                        click.echo("{:06f} {:06f} {:06f}".format(*[j**2.2 for j in c[0:3]]))
+                    
         except click.Abort:
             raise
         except Exception as ex:
@@ -619,7 +795,7 @@ def colors(**kwargs):
     return 'box', kwargs
 
 
-@main.resultcallback()
+@plot.resultcallback()
 @click.pass_context
 def printconfig(ctx, opts, **kwargs):
     """callback to save config file"""
@@ -630,3 +806,7 @@ def printconfig(ctx, opts, **kwargs):
     if kwargs['outconfig']:
         clk.print_config(ctx, opts, kwargs['outconfig'], kwargs['config'],
                          kwargs['configalias'])
+
+
+if __name__ == '__main__':
+    main()
\ No newline at end of file
