@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 # Copyright (c) 2018 Stephen Wasilewski
 # =======================================================================
 # This Source Code Form is subject to the terms of the Mozilla Public
@@ -8,12 +10,20 @@
 """functions for plotting with matplotlib"""
 
 from __future__ import print_function
+from __future__ import division
 
+from builtins import str
+from builtins import zip
+from builtins import range
+from builtins import object
+from past.utils import old_div
 import numpy as np
 import math
+import matplotlib
+matplotlib.use('Agg')
 import matplotlib.pyplot as plt
 from matplotlib import rcParams
-from matplotlib.patches import Circle
+from matplotlib.patches import Circle, Patch
 import matplotlib.colors as colors
 import matplotlib.cm as cmx
 from matplotlib.transforms import Bbox
@@ -65,10 +75,10 @@ def tick_from_arg(ax, xs, ys, a4, kwargs):
     try:
         a4.update(get_axes(kwargs['axes'], xs, ys, kwargs['polar'],
                            kwargs['polarauto'], kwargs['stacked'], pery=pery))
-    except Exception, ex:
+    except Exception as ex:
         try:
             a4.update(get_axes(kwargs['axes'], xs, ys, kwargs['polar'], stacked=kwargs['stacked'], pery=pery))
-        except Exception, ex:
+        except Exception as ex: 
             raise
             pass
     ax = ticks(ax, **a4)
@@ -140,7 +150,7 @@ def get_axes(arg, xs, ys, polar=False, polarauto=True, stacked=False, pery=False
     if polar and polarauto:
         naxes['xdata'] = [0, 2*math.pi]
     elif polar:
-        naxes['xdata'] = [axes[1]*math.pi/180, axes[2]*math.pi/180]
+        naxes['xdata'] = [old_div(axes[1]*math.pi,180), old_div(axes[2]*math.pi,180)]
     return naxes
 
 
@@ -173,12 +183,8 @@ def plot_setup(fg='black', bg='white', polar=False, areaonly=False, params={}):
     rcParams.update(defparam)
     fig = plt.figure()
     if areaonly:
-        if polar:
-            ax = fig.add_subplot(1, 1, 1, position=[0, 0, 1, 1], facecolor=bg,
-                                 polar=polar, frame_on=True)
-        else:
-            ax = fig.add_subplot(1, 1, 1, position=[0, 0, 1, 1], facecolor=bg,
-                                 polar=polar, frame_on=False)
+        ax = fig.add_subplot(1, 1, 1, position=[0, 0, 1, 1], facecolor=bg,
+                             polar=polar, frame_on=polar)
     else:
         ax = fig.add_subplot(1, 1, 1, facecolor=bg, polar=polar)
     return ax, fig
@@ -188,7 +194,8 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
           xgrid=False, ygrid=False, xscale='linear', yscale='linear',
           annualx=False, dayy=False, pery=False, ticklines=False, pph=1,
           bottom=None, bg='white', xlabels=None, dpy=365, hpd=24, sh=0,
-          polar=False, xticks=None, yticks=None, labelweight='bold'):
+          polar=False, xticks=None, yticks=None, labelweight='bold',
+          matchxy=False):
     """
     setup ticks/axes for plot
 
@@ -224,6 +231,11 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
     xmax = xdata[-1]
     ymin = min(ydata)
     ymax = max(ydata)
+    if matchxy:
+        xmin = min(xmin, ymin)
+        xmax = max(xmax, ymax)
+        ymin = xmin
+        ymax = xmax
     ax.xaxis.grid(linestyle="-", linewidth=0.3, color=tcol)
     ax.yaxis.grid(linestyle="-", linewidth=0.3, color=tcol, zorder=1)
     ax.xaxis.grid(xgrid)
@@ -259,13 +271,13 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
         ax.set_ylim(bottom=ylim[0], top=ylimb[1])
         ax.get_yaxis().get_major_formatter().set_useOffset(bottom)
     if xlabels is not None and len(xlabels) > 0:
-        inc = (xmax-xmin)/len(xlabels)
+        inc = old_div((xmax-xmin),len(xlabels))
         if polar:
             ax.set_xticks(np.arange(xmin, xmax, inc))
             ax.set_xticklabels(xlabels)
             ax.set_xlim(left=xmin, right=xmax)
         else:
-            ax.set_xticks(np.arange(xmin+inc/2, xmax+inc/2, inc))
+            ax.set_xticks(np.arange(xmin+old_div(inc,2), xmax+old_div(inc,2), inc))
             if max([len(i) for i in xlabels]) > 10:
                 ro = 'vertical'
             else:
@@ -289,9 +301,9 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
         bottom = 0
     if yticks is not None:
         ax.set_yticks(np.append(np.arange(bottom+ymin, ymax+bottom,
-                      (ymax-ymin)/(yticks)),ymax+bottom))
+                      old_div((ymax-ymin),(yticks))),ymax+bottom))
     if xticks is not None:
-        ax.set_xticks(np.append(np.arange(xmin, xmax, (xmax-xmin)/(xticks)),xmax))
+        ax.set_xticks(np.append(np.arange(xmin, xmax, old_div((xmax-xmin),(xticks))),xmax))
     return ax
 
 
@@ -305,20 +317,20 @@ def plot_legend(handles, bbox_to_anchor=(1.05, 1), loc=2,
 
 
 def add_colorbar(fig, pc, axes=[0.3, 0.0, 0.4, 0.02],
-                 ticks=None, ticklabels=None):
+                 ticks=None, ticklabels=None, orientation='horizontal'):
     """add colorbar scale to figure below plot"""
     cbaxes = fig.add_axes(axes)
     if ticklabels is not None:
         if ticks is None:
             vmin, vmax = pc.get_clim()
             ticks = np.linspace(vmin, vmax, len(ticklabels))
-        cb = fig.colorbar(pc, orientation="horizontal",
+        cb = fig.colorbar(pc, orientation=orientation,
                           cax=cbaxes, ticks=ticks)
         if ticklabels is not None:
             cb.ax.set_xticklabels(ticklabels)
     else:
-        cb = fig.colorbar(pc, orientation="horizontal", cax=cbaxes)
-    cb.ax.tick_params(labelsize=8)
+        cb = fig.colorbar(pc, orientation=orientation, cax=cbaxes)
+    cb.ax.tick_params(labelsize=8, width=.5)
     cb.outline.set_linewidth(.5)
     for t in cb.ax.get_xticklines():
         t.set_visible(False)
@@ -326,69 +338,34 @@ def add_colorbar(fig, pc, axes=[0.3, 0.0, 0.4, 0.02],
 
 def plot_graph(fig, saveimage, width=10.5, height=5, bg='white', fg='black',
                handles=[], dpi=200, bbox_to_anchor=(1.05, 1),
-               loc=2, legend=False, format='png', background=None,
+               loc=2, legend=False, background=None,
                front=False, alpha=.5, areaonly=False, polar=False):
     """add legend and save image of plot"""
     fig.set_size_inches(width, height)
     if legend and not areaonly:
         plot_legend(handles=handles, bbox_to_anchor=bbox_to_anchor,
                     loc=loc, bg=bg, fg=fg)
-    if polar:
-        if background is not None:
-            im = plt.imread(background)
-            ax = fig.axes[0]
-            ax2 = fig.add_axes([0, 0, 1, 1])
-            extent = ax2.get_xlim() + ax2.get_ylim()
-            plt.imshow(im, extent=extent, alpha=alpha, aspect='auto')
-            if front:
-                ax.set_zorder(ax2.get_zorder()-1)
-            else:
-                ax.set_zorder(ax2.get_zorder()+1)
-                ax.patch.set_visible(False)
-            ax2.axis('off')
-        if areaonly:
-            ax = fig.axes[0]
-            ax.axes.set_xlabel("")
-            ax.axes.set_ylabel("")
-            ax.xaxis.set_ticklabels([])
-            ci = [i.get_extents() for i in ax.get_children()
-                  if type(i) == Circle]
-            extent = Bbox([[min([i.xmin for i in ci]),
-                            min([i.ymin for i in ci])],
-                           [max([i.xmax for i in ci]),
-                            max([i.ymax for i in ci])]])
-            wi = width*(extent.xmax-extent.xmin)/500
-            he = height*(extent.ymax-extent.ymin)/500
-            ow = width*extent.xmin/500
-            oh = height*extent.ymin/500
-            extent = Bbox([[ow, oh], [wi+ow, he+oh]])
-            plt.savefig(saveimage, dpi=dpi, bbox_inches=extent, aspect='auto',
-                        facecolor=bg, pad_inches=0)
-        else:
-            plt.savefig(saveimage, dpi=dpi, bbox_inches='tight', facecolor=bg)
+    ax = fig.axes[0]
+    if areaonly:
+        ax.axes.set_xlabel("")
+        ax.axes.set_ylabel("")
+        ax.xaxis.set_ticklabels([])
+        ax2 = fig.add_axes([0, 0, 1, 1])
+        plargs = dict(bbox_inches=None, aspect='auto', pad_inches=0)
     else:
-        if areaonly:
-            ax = fig.axes[0]
-            if background is not None:
-                im = plt.imread(background)
-                ax.imshow(im, extent=ax.get_xlim() + ax.get_ylim(),
-                          alpha=alpha, aspect='auto')
-            plt.savefig(saveimage, dpi=dpi, bbox_inches=None, aspect='auto',
-                        facecolor=bg, pad_inches=0)
-        else:
-            if background is not None:
-                im = plt.imread(background)
-                ax = fig.axes[0]
-                ax2 = ax.twinx()
-                plt.imshow(im, extent=ax2.get_xlim() + ax2.get_ylim(),
-                           alpha=alpha, aspect='auto')
-                if front:
-                    ax.set_zorder(ax2.get_zorder()-1)
-                else:
-                    ax.set_zorder(ax2.get_zorder()+1)
-                    ax.patch.set_visible(False)
-                ax2.axis('off')
-            plt.savefig(saveimage, dpi=dpi, bbox_inches='tight', facecolor=bg)
+        ax2 = fig.add_subplot(1, 1, 1, label='background')
+        plargs = dict(bbox_inches='tight')
+    extent = ax2.get_xlim() + ax2.get_ylim()
+    if background is not None:
+        im = plt.imread(background)
+        plt.imshow(im, extent=extent, alpha=alpha, aspect='auto')
+    if front:
+        ax.set_zorder(ax2.get_zorder()-1)
+    else:
+        ax.set_zorder(ax2.get_zorder()+1)
+        ax.patch.set_visible(False)
+    ax2.axis('off')
+    plt.savefig(saveimage, dpi=dpi, facecolor=bg, **plargs)
     plt.close()
 
 
@@ -420,7 +397,7 @@ def get_colors(cmap, step=None, positions=None, funcs=[]):
         try:
             colormap = func(cmap, step, positions)
             break
-        except Exception, err:
+        except Exception as err:
             e.append(str(err))
             continue
     else:
@@ -526,10 +503,11 @@ def plot_criteria(ax, x, y, criteria, flipxy=False,
         ax.plot(filtx, filty, **kwargs)
 
 
-def plot_scatter(ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
+def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
                  mrk='o', step=None, fcol=0.0, mew=0.0, emap=None,
                  flipxy=False, cs=None, cmin=None, cmax=None,
-                 msd=None, mmin=None, mmax=None, **kwargs):
+                 msd=None, mmin=None, mmax=None, legend=True, polar=False,
+                 **kwargs):
     """adds scatterplots/lines to ax and returns ax and handles for legend"""
     nlab = len(labels)
     for i in range(len(ys)):
@@ -537,10 +515,10 @@ def plot_scatter(ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
             labels.append("series{:02d}".format(i))
     handles = []
     if step is not None:
-        inc = (1.-fcol)/(step-1)
+        inc = old_div((1.-fcol),(step-1))
     else:
         try:
-            inc = (1.-fcol)/(len(ys)-1)
+            inc = old_div((1.-fcol),(len(ys)-1))
         except ZeroDivisionError:
             inc = 1
     if emap is None:
@@ -574,6 +552,12 @@ def plot_scatter(ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
                         'norm': colormap.norm}
             plotargs.update(kwargs)
             ax.scatter(x, y, **plotargs)
+            if legend:
+                pc = cmx.ScalarMappable(norm=colormap.norm, cmap=colormap.cmap)
+                if polar:
+                    add_colorbar(fig, colormap, axes = [1, 0.11, .02, 0.77], orientation='vertical')
+                else:
+                    add_colorbar(fig, colormap, axes = [.92, 0.11, .02, 0.77], orientation='vertical')
         elif msd is not None:
             plotargs = {'linewidth': lwa, 's': msa**2, 'label': l,
                         'marker': mka, 'linewidth': mewa, 'c': [c],
@@ -603,9 +587,9 @@ def plot_heatmap(fig, ax, data, colormap, vmin=None, vmax=None,
     """adds heatmap and colorbar to ax returns ax"""
     xlim = ax.axes.get_xlim()
     xrng = abs(xlim[1]-xlim[0])
-    ppd = int(len(data)/xrng)
+    ppd = int(old_div(len(data),xrng))
     if dst:
-        data = shift_data(data, ppd/hpd, hpd=hpd, dpy=dpy, sh=sh)
+        data = shift_data(data, old_div(ppd,hpd), hpd=hpd, dpy=dpy, sh=sh)
     try:
         mtx = np.reshape(data, (int(xrng), ppd))
         tmtx = np.transpose(mtx)
@@ -648,16 +632,16 @@ def plot_bar(ax, xs, ys, labels, colormap, stacked=False, rwidth=.8, step=None,
     xlim = [xl[0]+brng[0]*(xl[1]-xl[0]), xl[0]+brng[1]*(xl[1]-xl[0])]
     xrng = abs(xlim[1]-xlim[0])/float(len(xs[0]))
     lxs = len(xs[0])
-    width = xrng/nlab*rwidth
+    width = old_div(xrng,nlab)*rwidth
     try:
-        xsc = (lxs - 1.0)/lxs
+        xsc = old_div((lxs - 1.0),lxs)
     except ZeroDivisionError:
         xsc = 1
     xsc = xsc * (brng[1] - brng[0])
     if polar and polar0:
         off0 = brng[0] * abs(xl[1]-xl[0])
     else:
-        off0 = xrng/2-width*(nlab-1)/2 + xlim[0]
+        off0 = old_div(xrng,2)-old_div(width*(nlab-1),2) + xlim[0]
     w2 = width*bwidth
     bot = 0
     for i, (x, y, l) in enumerate(zip(xs, ys, labels)):
@@ -678,35 +662,106 @@ def plot_bar(ax, xs, ys, labels, colormap, stacked=False, rwidth=.8, step=None,
     return ax, handles
 
 
-def plot_box(ax, data, labels, colormap, ylim, rwidth=.8,
-               step=None, **kwargs):
+def plot_box(ax, data, labels, colormap, ylim, rwidth=.8, step=None, mark='x',
+             mew=0.5, ms=3.0, lw=1.0, clw=1.0, clbg=True, fillalpha=1.0, notch=False,
+             series=1, bg='white', inline=False, mean=False, **kwargs):
     """adds box plots to ax and returns ax and handles for legend"""
     nlab = len(labels)
-    for i in range(len(data)):
+    for i in range(series):
         if i >= nlab:
             labels.append("series{:02d}".format(i))
-    try:
-        inc = 1./(len(data)-1)
-    except ZeroDivisionError:
-        inc = 1
-    c = [colormap.to_rgba(i*inc) for i in range(len(data))]
-    plotargs = {
-        'boxprops' : {'linewidth':0,'color':c[0]},
-        'capprops' : {'color':c[0],'linewidth':2,'solid_capstyle':'butt'},
-        'whiskerprops' : {'color':c[0],'linewidth':2,'solid_capstyle':'butt'},
-        'medianprops' : {'linewidth':2,'color':'white','solid_capstyle':'butt'}
-    }
-    plotargs.update(kwargs)
-    boxplot = ax.boxplot(data, patch_artist=True, widths=rwidth, whis='range', bootstrap=1000,
-                         **plotargs)
-    handles, labels = ax.get_legend_handles_labels()
-    # if ylim[1] is None:
-    #     ymax = max(flat(boxplot[0]))*1.1
-    # else:
-    #     ymax = ylim[1]
-    # ymin = ylim[0]
-    # ax.set_ylim(bottom=ymin, top=ymax)
-    # ax.set_xlim(left=min(boxplot[1]), right=max(boxplot[1]))
+    nlab = len(labels)
+    if step is not None:
+        inc = 1./(step-1)
+    else:
+        try:
+            inc = 1./(series-1)
+        except ZeroDivisionError:
+            inc = 1
+    chunksize = int(len(data)/series)
+    handles = []
+    
+    for i in range(series):
+        c = colormap.to_rgba(i*inc)
+        if clbg:
+            medianc = bg
+        else:
+            medianc = c
+        facecolor = c[0:3] + (fillalpha,)
+        plotargs = {
+            'boxprops' : {'linewidth':lw,'facecolor':facecolor, 'color':c},
+            'capprops' : {'color':c,'linewidth':lw,'solid_capstyle':'butt'},
+            'whiskerprops' : {'color':c,'linewidth':lw,'solid_capstyle':'butt'},
+            'medianprops' : {'linewidth':clw*(not mean),'color':medianc,'solid_capstyle':'butt'},
+            'meanprops' : {'linewidth':clw,'color':medianc,
+                           'solid_capstyle':'butt', 'ls':'-', 'marker':'o',
+                           'ms':clw*2, 'mfc':medianc, 'mew':0},
+            'flierprops' : {'marker':mark, 'markeredgecolor':c, 'markeredgewidth':mew, 'markersize':ms}
+        }
+        plotargs.update(kwargs)
+        if inline:
+            x = np.arange(2, len(data), series)
+            sw = series
+        else:
+            x = np.arange(i, len(data), series)
+            sw = 1
+        boxplot = ax.boxplot(data[i*chunksize:i*chunksize+chunksize], notch=notch,
+                             patch_artist=True, widths=rwidth*sw, bootstrap=1000,
+                             positions=x , manage_ticks=False, showmeans=mean, meanline=not notch,
+                             **plotargs)
+        handles.append(Patch(color=c, label=labels[i]))
+    lims = ax.dataLim
+    ax.set_xlim(left=lims.x0-rwidth, right=lims.x1+rwidth)
+    return ax, handles
+
+
+def plot_violin(ax, data, labels, colormap, ylim, rwidth=.8, step=None, lw=1.0,
+                clw=1.0, clbg=True, fillalpha=1.0, median=True,
+                series=1, bg='white', inline=False, mean=False, **kwargs):
+    """adds violin plots to ax and returns ax and handles for legend"""
+    nlab = len(labels)
+    for i in range(series):
+        if i >= nlab:
+            labels.append("series{:02d}".format(i))
+    nlab = len(labels)
+    if step is not None:
+        inc = 1./(step-1)
+    else:
+        try:
+            inc = 1./(series-1)
+        except ZeroDivisionError:
+            inc = 1
+    chunksize = int(len(data)/series)
+    handles = []
+    for i in range(series):
+        c = colormap.to_rgba(i*inc)
+        if clbg:
+            medianc = bg
+        else:
+            medianc = c
+        if inline:
+            x = np.arange(2, len(data), series)
+            sw = series
+        else:
+            x = np.arange(i, len(data), series)
+            sw = 1
+        
+        vplot = ax.violinplot(data[i*chunksize:i*chunksize+chunksize],
+                              showmeans=mean, showmedians=median,
+                              widths=rwidth*sw, positions=x)
+        for vp in vplot['bodies']:
+            vp.set_facecolor(c)
+            vp.set_alpha(fillalpha)
+        for vp in [ vplot[i] for i in ['cmins', 'cmaxes', 'cbars', 'cmeans']]:
+            vp.set_edgecolor(c)
+            vp.set_linewidth(lw)
+        for vp in [ vplot[i] for i in ['cmedians']]:
+            vp.set_edgecolor(c)
+            vp.set_linewidth(clw)
+        vplot['cmeans'].set_linestyle(':')
+        handles.append(Patch(color=c, label=labels[i]))
+    lims = ax.dataLim
+    ax.set_xlim(left=lims.x0-rwidth, right=lims.x1+rwidth)
     return ax, handles
 
 
@@ -906,7 +961,7 @@ class CliptColors(object):
                     (194, 214, 154), (215, 228, 188)]
         }
         self.cdict = {}
-        for k, v in self.cdict255.iteritems():
+        for k, v in self.cdict255.items():
             self.cdict[k] = [tuple(i/255. for i in j) for j in v]
         self.colors = ['blu', 'org', 'pur', 'red', 'blg',
                        'mag', 'brn', 'grn', 'yel', 'ggr']
@@ -981,7 +1036,7 @@ def cmap_from_clipt(cmap, step=None, positions=None):
     try:
         clist = clcol[cmap]
     except Exception:
-        clopts = ", ".join(clcol.cdict.keys())
+        clopts = ", ".join(list(clcol.cdict.keys()))
         note = "Additional clipt colors values:"
         message = "{}\n\n{}".format(note, clopts)
         raise ValueError(message)
